/*
 * (C) Copyright 2010-2017 Redwood Technology B.V., Houten, The Netherlands
 *
 * $Id$
 */
package com.redwood.scheduler.api.database.examples;

import java.io.PrintWriter;
import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Arrays;

import com.redwood.scheduler.api.database.DatabaseConnection;
import com.redwood.scheduler.api.database.Null;
import com.redwood.scheduler.api.database.OutputParameter;
import com.redwood.scheduler.api.exception.LicenseException;
import com.redwood.scheduler.api.job.UserJobContext;
import com.redwood.scheduler.api.model.Credential;
import com.redwood.scheduler.api.model.CredentialProtocol;
import com.redwood.scheduler.api.model.Database;
import com.redwood.scheduler.api.model.SchedulerSession;

/**
 * Examples of using the database APIs, see
 * <a href="Examples.java">Examples.java</a> for the source.
 */
public class Examples
{
  public void examples(UserJobContext jc, SchedulerSession jcsSession)
    throws SQLException, LicenseException
  {
    try (PrintWriter jcsOut = jc.getOutputWriter())
    {
      // Get the credentials
      final Credential credential = getJdbcCredential(jcsSession, "aUser", "anEndpoint");
      // Get the Database to connect to
      final Database db = jcsSession.getDatabaseByName("aDatabase");
      try (final DatabaseConnection dbc = db.connect(credential))
      {

        // Example 1: Query using inversion of control
        dbc.executeQuery("select column from table", null, rs1 ->
          {
            while (rs1.next())
            {
              jcsOut.write(rs1.getString(1) + "\n");
            }
          });

        // Example 2: Query using auto close result set
        try (ResultSet rs2 = dbc.executeQueryAutoClose("select column from table"))
        {
          while (rs2.next())
          {
            jcsOut.write(rs2.getString(1) + "\n");
          }
        }

        // Example 3: Query using input parameters
        try (ResultSet rs3 = dbc.executeQueryAutoClose("select column from table where key = ?", Long.valueOf(3)))
        {
          while (rs3.next())
          {
            jcsOut.write(rs3.getString(1) + "\n");
          }
        }

        // Example 4: Stored procedure call using input parameters (one of which is
        // null) and output parameters
        // The {call ...} syntax is JDBC specific it will be translated by the
        // driver.
        OutputParameter outPar3 = new OutputParameter(Types.BIGINT);
        dbc.executeUpdate("{call procedure_name(?, ?, ...)}", Null.VARCHAR, new BigDecimal(7), outPar3);

        BigDecimal outPar3Value = (BigDecimal) outPar3.getValue();
        jcsOut.println("The output parameter value is " + outPar3Value);

        // Example 5: Same as Example 4, but using the wrapped JDBC API
        try (Connection con = dbc.getWrappedConnectionAutoClose())
        {
          try (CallableStatement cs = con.prepareCall("{call procedure_name(?, ?, ...)}"))
          {
            cs.setNull(1, Types.VARCHAR);
            cs.setLong(2, Types.BIGINT);
            cs.registerOutParameter(3, Types.BIGINT);
            cs.execute();
            cs.getInt(3);
          }
        }

        // Example 6: Getting the unwrapped connection. This will always log a
        // message to stderr.log that the resource cleanup is not guaranteed.
        // Connection unwrapped = (Connection) ((Unwrap) con).unwrap();
        //
        // Use the unwrapped connection
        // ((OracleConnection) unwrapped).setClientIdentifier("MyClientID");
      }
    }
  }

  /**
   * An example of Database.createDatabaseConnectionBuilder
   *
   * @param jc
   * @param jcsSession
   * @throws SQLException
   * @throws LicenseException
   *
   * @see #examples(UserJobContext, SchedulerSession)
   */
  public void exampleUsingBuilder(UserJobContext jc, SchedulerSession jcsSession)
    throws SQLException, LicenseException
  {
    try (PrintWriter jcsOut = jc.getOutputWriter())
    {
      // Get the credentials
      final Credential credential = getJdbcCredential(jcsSession, "aUser", "anEndpoint");
      // Get the Database to connect to
      final Database db = jcsSession.getDatabaseByName("aDatabase");
      try (final DatabaseConnection con = //
          db.createDatabaseConnectionBuilder() //
            .setPreLogger((sql, parameters) -> jcsOut.println("Executing " + sql + " using " + Arrays.toString(parameters))) //
            .setPostLogger((sql, parameters) -> jcsOut.println("Succesfully executed " + sql + " using " + Arrays.toString(parameters))) //
            .setErrorLogger((cause, sql, parameters) ->
              {
                jcsOut.println("Failed to execute " + sql + " using " + Arrays.toString(parameters) + ", error " + cause.getMessage());
              }) //
            .connect(credential) //
      )
      {
        // Now you can use the connection again, like the examples in method
        // examples(UserJobContext, SchedulerSession).
      }
    }

  }

  private Credential getJdbcCredential(final SchedulerSession jcsSession, final String username, final String endpoint)
  {
    // Get the JDBC CredentialProtocol
    final CredentialProtocol protocol = jcsSession.getCredentialProtocolByName(CredentialProtocol.JDBC);

    // Get the credentials
    return jcsSession.getCredentialByCredentialProtocolEndpointRealUser(protocol, username, endpoint);
  }
}
